@@ -38,11 +38,38 @@
 #include "vicious.h"
 
 #include "gdm.h"
+#include "gdmcommon.h"
 #include "gdmcomm.h"
 #include "gdmconfig.h"
 
-static gboolean debug = FALSE;
-static int num_cmds   = 0;
+static gboolean bulk_acs = FALSE;
+static gboolean debug    = FALSE;
+static gboolean quiet    = FALSE;
+static int      num_cmds = 0;
+
+/*
+ * Note, in this function we have to call gdm_common_error instead
+ * of gdm_common_debug, since gdm_common_debug accesses the
+ * GDM_KEY_DEBUG which can cause a sockets connection to get the
+ * config data, causing an infinite loop.  This is why clients must
+ * call the gdmcomm_set_debug function to turn on debug for these
+ * functions.  
+ */
+void
+gdmcomm_set_debug (gboolean enable)
+{
+	debug = enable;
+}
+
+/*
+ * Normally errors are printed.  Setting quiet to TRUE turns off
+ * display of error messages.
+ */
+void
+gdmcomm_set_quiet_errors (gboolean enable)
+{
+	quiet = enable;
+}
 
 static char *
 do_command (int fd, const char *command, gboolean get_response)
@@ -55,8 +82,9 @@ do_command (int fd, const char *command, gboolean get_response)
 	void (*old_handler)(int);
 #endif
 
-	if (debug)
-		g_print ("Sending command: '%s'\n", command);
+	if (debug) {
+		gdm_common_error ("Sending command: '%s'", command);
+	}
 
 	cstr = g_strdup_printf ("%s\n", command);
 
@@ -70,12 +98,14 @@ do_command (int fd, const char *command, gboolean get_response)
 	g_free (cstr);
 
 	num_cmds++;
-	if (debug)
-		g_print ("Incrementing num_cmds to %d\n", num_cmds);
 
-	if (ret < 0)
+	if (ret < 0) {
+		if ( !quiet)
+			gdm_common_error ("Command failed, no data returned");
 		return NULL;
+	}
 
+	/* No need to print debug, this is only used when closing */
 	if ( ! get_response)
 		return NULL;
 
@@ -86,12 +116,25 @@ do_command (int fd, const char *command, gboolean get_response)
 	}
 
 	if (debug)
-		g_print ("  Got response: '%s'\n", str->str);
+		gdm_common_error ("  Got response: '%s'", str->str);
 
 	cstr = str->str;
 	g_string_free (str, FALSE);
 
-	if (strcmp (ve_sure_string (cstr), "ERROR 200 Too many messages") == 0) { 
+	/*
+	 * If string is empty, then the daemon likely closed the connection 
+	 * because of too many subconnections.  At any rate the daemon should
+	 * not return an empty string.  All return values should start with
+	 * "OK" or "ERROR".  Daemon should never complain about too many
+	 * messages since the slave keeps track of the number of commands sent
+	 * and should not send too many, but it does not hurt to check and
+	 * manage it if it somehow happens.  In either case return NULL
+	 * instead so the caller can try again.
+	 */
+	if (ve_string_empty (cstr) ||
+	    strcmp (ve_sure_string (cstr), "ERROR 200 Too many messages") == 0) {
+		if ( !quiet)
+			gdm_common_error ("Command failed, daemon busy.");
 		g_free (cstr);
 		return NULL;
 	}
@@ -119,55 +162,126 @@ version_ok_p (const char *version, const char *min_version)
 		return FALSE;
 }
 
-char *
-gdmcomm_call_gdm (const char *command, const char * auth_cookie, const char *min_version, int tries)
+static gboolean allow_sleep          = TRUE;
+static gboolean did_sleep_on_failure = FALSE;
+static int comm_fd                   = 0;
+
+static char *
+gdmcomm_call_gdm_real (const char *command,
+		       const char *auth_cookie,
+		       const char *min_version,
+		       int tries,
+		       int try_start)
 {
-	static int fd = 0;
 	char *ret;
 
-	if (num_cmds == (GDM_SUP_MAX_CONNECTIONS)) {
-	   do_command (fd, GDM_SUP_CLOSE, FALSE);
-	   VE_IGNORE_EINTR (close (fd));
-	   fd       = 0;
+	/*
+         * If already sent the max number of commands, close the connection
+         * and reopen.  Subtract 1 to allow the "CLOSE" to get through.
+         */
+	if (num_cmds == (GDM_SUP_MAX_MESSAGES - 1)) {
+	   if (debug)
+		gdm_common_error ("  Closing and reopening connection.");
+	   do_command (comm_fd, GDM_SUP_CLOSE, FALSE);
+	   VE_IGNORE_EINTR (close (comm_fd));
+	   comm_fd  = 0;
 	   num_cmds = 0;
 	}
 
-	if (tries <= 0)
+	if (tries <= 0) {
+		if ( !quiet)
+			gdm_common_error ("  Command failed %d times, aborting.", try_start);
 		return NULL;
+	}
 
-	if (fd <= 0) {
+	if (!quiet && tries != try_start) {
+		gdm_common_error ("  Trying failed command again.  Try %d of %d.",
+			(try_start - tries + 1), try_start);
+	}
+
+	if (comm_fd <= 0) {
 		struct sockaddr_un addr;
 		strcpy (addr.sun_path, GDM_SUP_SOCKET);
 		addr.sun_family = AF_UNIX;
+		comm_fd = socket (AF_UNIX, SOCK_STREAM, 0);
+		if (comm_fd < 0) {
+			if ( !quiet)
+				gdm_common_error ("  Failed to open socket");
 
-		fd = socket (AF_UNIX, SOCK_STREAM, 0);
-		if (fd < 0) {
-			return gdmcomm_call_gdm (command, auth_cookie, min_version, tries - 1);
+			return gdmcomm_call_gdm_real (command, auth_cookie, min_version, tries - 1, try_start);
 		}
 
-		if (connect (fd, (struct sockaddr *)&addr, sizeof (addr)) < 0) {
-			VE_IGNORE_EINTR (close (fd));
-			fd = 0;
-			return gdmcomm_call_gdm (command, auth_cookie, min_version, tries - 1);
+		if (connect (comm_fd, (struct sockaddr *)&addr, sizeof (addr)) < 0) {
+
+
+			/*
+			 * If there is a failure on connect, there are probably
+                         * other clients fighting for the connection, so sleep
+                         * for 1 second before retry to avoid failing over and
+                         * over in a tight loop.
+                         *
+                         * Only do this if allow_sleep is true.  allow_sleep
+                         * will get set to FALSE if the first call to this 
+                         * function fails all retries.
+			 */
+			if (allow_sleep == TRUE) {
+
+				did_sleep_on_failure = TRUE;
+
+				/*
+				 * Only actualy sleep if we are going to try
+				 * again.
+				 */
+				if (tries > 1) {
+					if ( !quiet)
+						gdm_common_error ("  Failed to connect to socket, sleep 1 second and retry");
+					sleep (1);
+				}
+			} else {
+				if ( !quiet)
+					gdm_common_error ("  Failed to connect to socket, not sleeping");
+			}
+			VE_IGNORE_EINTR (close (comm_fd));
+			comm_fd = 0;
+			return gdmcomm_call_gdm_real (command, auth_cookie,
+				min_version, tries - 1, try_start);
 		}
 
+		/*
+                 * If we get this far, then even if we did sleep in the past,
+		 * we did get a connection, so no need to prevent future
+		 * sleeps if required.
+                 */
+		allow_sleep          = TRUE;
+                did_sleep_on_failure = FALSE;
+
 		/* Version check first - only check first time */
-		ret = do_command (fd, GDM_SUP_VERSION, TRUE /* get_response */);
+		ret = do_command (comm_fd, GDM_SUP_VERSION, TRUE);
 		if (ret == NULL) {
-			VE_IGNORE_EINTR (close (fd));
-			fd = 0;
-			return gdmcomm_call_gdm (command, auth_cookie, min_version, tries - 1);
+			if ( !quiet)
+				gdm_common_error ("  Version check failed");
+			VE_IGNORE_EINTR (close (comm_fd));
+			comm_fd = 0;
+			return gdmcomm_call_gdm_real (command, auth_cookie,
+				min_version, tries - 1, try_start);
 		}
 		if (strncmp (ret, "GDM ", strlen ("GDM ")) != 0) {
+			if ( !quiet)
+				gdm_common_error ("  Version check failed, bad name");
+
 			g_free (ret);
-			VE_IGNORE_EINTR (close (fd));
-			fd = 0;
+			do_command (comm_fd, GDM_SUP_CLOSE, FALSE);
+			VE_IGNORE_EINTR (close (comm_fd));
+			comm_fd = 0;
 			return NULL;
 		}
 		if ( ! version_ok_p (&ret[4], min_version)) {
+			if ( !quiet)
+				gdm_common_error ("  Version check failed, bad version");
 			g_free (ret);
-			VE_IGNORE_EINTR (close (fd));
-			fd = 0;
+			do_command (comm_fd, GDM_SUP_CLOSE, FALSE);
+			VE_IGNORE_EINTR (close (comm_fd));
+			comm_fd = 0;
 			return NULL;
 		}
 		g_free (ret);
@@ -177,34 +291,115 @@ gdmcomm_call_gdm (const char *command, const char * auth_cookie, const char *min
 	if (auth_cookie != NULL)  {
 		char *auth_cmd = g_strdup_printf
 			(GDM_SUP_AUTH_LOCAL " %s", auth_cookie);
-		ret = do_command (fd, auth_cmd, TRUE /* get_response */);
+		ret = do_command (comm_fd, auth_cmd, TRUE);
 		g_free (auth_cmd);
 		if (ret == NULL) {
-			VE_IGNORE_EINTR (close (fd));
-			fd = 0;
-			return gdmcomm_call_gdm (command, auth_cookie,
-						 min_version, tries - 1);
+			VE_IGNORE_EINTR (close (comm_fd));
+			comm_fd = 0;
+			return gdmcomm_call_gdm_real (command, auth_cookie,
+				min_version, tries - 1, try_start);
 		}
 		/* not auth'ed */
 		if (strcmp (ve_sure_string (ret), "OK") != 0) {
-			VE_IGNORE_EINTR (close (fd));
-			fd = 0;
+			if ( !quiet)
+				gdm_common_error ("  Error, auth check failed");
+			do_command (comm_fd, GDM_SUP_CLOSE, FALSE);
+			VE_IGNORE_EINTR (close (comm_fd));
+			comm_fd = 0;
 			/* returns the error */
 			return ret;
 		}
 		g_free (ret);
 	}
 
-	ret = do_command (fd, command, TRUE /* get_response */);
+	ret = do_command (comm_fd, command, TRUE);
 	if (ret == NULL) {
-		VE_IGNORE_EINTR (close (fd));
-		fd = 0;
-		return gdmcomm_call_gdm (command, auth_cookie, min_version, tries - 1);
+		VE_IGNORE_EINTR (close (comm_fd));
+		comm_fd = 0;
+		return gdmcomm_call_gdm_real (command, auth_cookie,
+			min_version, tries - 1, try_start);
+	}
+
+	/*
+	 * We want to leave the connection open if bulk_acs is set to
+	 * true, so clients can read as much config data in one 
+	 * sockets connection when it is set.  This requires that
+	 * GDM client programs ensure that they call the bulk_start
+	 * and bulk_stop functions around blocks of code that
+	 * need to read data in bulk.  If a client reads config data
+	 * outside of the bulk_start/stop functions, then this
+	 * will just negatively affect performance since an additional
+	 * socket will be opened to read that config data.
+	 */
+	if (bulk_acs == FALSE) {
+		do_command (comm_fd, GDM_SUP_CLOSE, FALSE);
+		VE_IGNORE_EINTR (close (comm_fd));
+		comm_fd = 0;
 	}
 
 	return ret;
 }
 
+char *
+gdmcomm_call_gdm (const char *command, const char * auth_cookie,
+	const char *min_version, int tries)
+{
+
+	char *retstr;
+
+	retstr = gdmcomm_call_gdm_real (command, auth_cookie, min_version,
+		tries, tries);
+
+	/*
+	 * Disallow sleeping on future calls if it failed to connect.
+         * did_sleep_on_failure will only be TRUE if the function returned
+	 * without ever connecting.
+	 */
+	if (did_sleep_on_failure == TRUE)
+		allow_sleep = FALSE;
+
+	return (retstr);
+}
+
+/**
+ * gdmcomm_did_connection_fail
+ *
+ * If allow_sleep is TRUE, then connection was able to go through.
+ * so the client can call this function after calling to see if
+ * the failure was due to the connection being too busy.  This is
+ * useful for gdmdynamic.
+ */
+gboolean
+gdmcomm_did_connection_fail (void)
+{
+	return !allow_sleep;
+}
+
+void
+gdmcomm_set_allow_sleep (gboolean val)
+{
+	allow_sleep = val;
+}
+
+void
+gdmcomm_comm_bulk_start (void)
+{
+	bulk_acs = TRUE;
+}
+
+void
+gdmcomm_comm_bulk_stop (void)
+{
+	/* Close the connection */
+	if (comm_fd > 0) {
+		do_command (comm_fd, GDM_SUP_CLOSE, FALSE);
+		VE_IGNORE_EINTR (close (comm_fd));
+	}
+	comm_fd  = 0;
+	num_cmds = 0;
+	bulk_acs = FALSE;
+}
+
 const char *
 gdmcomm_get_display (void)
 {
@@ -514,8 +709,3 @@ gdmcomm_get_error_message (const char *ret, gboolean use_xnest)
 	}
 }
 
-void
-gdmcomm_set_debug (gboolean enable)
-{
-	debug = enable;
-}
